use leptos::prelude::*;
use leptos_meta::*;
use leptos_use::{use_timestamp};
use chrono::{prelude::{DateTime, Local, Timelike}, NaiveDateTime};
use calendrier::{DateTime as FRDateTime, Timestamp as FRTimestamp};
use serde_json::Value;

pub mod wmo;

#[component]
pub fn Weather() -> impl IntoView {
    let weather_json = LocalResource::new(async move || {
        let json_response = reqwest::get("https://api.open-meteo.com/v1/forecast?latitude=60.236610&longitude=25.083630&timezone=auto&forecast_hours=17&wind_speed_unit=kn&hourly=temperature_2m,wind_speed_10m,precipitation,precipitation_probability,weather_code,is_day").await
            .and_then(|response| Ok(response.json::<Value>()));
        match json_response {
            Ok(result) => Some(result.await.ok()),
            _ => None
        }.flatten()
    });
    let get_hourly_slice = move |key: &str, start_index: usize, end_index: usize| {
        Some(weather_json.get().flatten()?["hourly"][key].as_array()?[start_index..end_index].to_vec().into_iter())
    };

    let times = move || {
        Some(get_hourly_slice("time", 0, 16)?.map(|timestamp| {
            let datetime = NaiveDateTime::parse_from_str(timestamp.as_str().unwrap_or_default(), "%Y-%m-%dT%H:%M");
            view! {<td>{format!("{}", datetime.unwrap_or_default().hour())}</td>}
        }).collect::<Vec<_>>())
    };
    let weather_codes = move || {
        let is_day = get_hourly_slice("is_day", 0, 16)?;
        let codes = get_hourly_slice("weather_code", 0, 16)?;
        Some(is_day.zip(codes).map(move |(is_day, code)| {
            let wmo_codes = crate::wmo::get_codes();
            let code_info = wmo_codes[code.to_string()][if is_day == Value::Number(1.into()) {"day"} else {"night"}].clone();
            let image = code_info["image"].as_str().unwrap_or_default();
            let description = code_info["description"].as_str().unwrap_or_default();
            view! { <td><img src={image.to_string()} alt={description.to_string()}/></td> }
        }).collect::<Vec<_>>())
    };
    let temperatures = move || {
        Some(get_hourly_slice("temperature_2m", 0, 16)?.map(|temperature| {
            view! { <td>{format!("{}°C", temperature.as_f64().unwrap_or_default().round().to_string())}</td> }
        }).collect::<Vec<_>>())
    };
    let wind = move || {
        Some(get_hourly_slice("wind_speed_10m", 0, 16)?.map(|wind_speed| {
            view! { <td>{format!("{}kn", wind_speed.as_f64().unwrap_or_default().round().to_string())}</td> }
        }).collect::<Vec<_>>())
    };
    let precipitation = move || {
        let amounts = get_hourly_slice("precipitation", 1, 17)?;
        let probabilities = get_hourly_slice("precipitation_probability", 1, 17)?;
        Some(amounts.zip(probabilities).map(|(amount, probability)| {
            Some(view! {
                <td>
                    <span>{format!("{}mm", amount.to_string())}</span>
                    <br/>
                    <span>{format!("{}%", probability)}</span>
                </td>
            })
        }).collect::<Vec<_>>())
    };

    view! {
        <div class="weather">
            <Suspense fallback=move || view! { <p>"Loading..."</p> }>
                <div class="forecast-per-hours">
                    <table class="forecast-per-hours">
                        <tr class="time">
                            <th>klo</th>
                            {times}
                        </tr>
                        <tr class="weather-code">
                            <th>sää</th>
                            {weather_codes}
                        </tr>
                        <tr class="temperature">
                            <th>lämpö</th>
                            {temperatures}
                        </tr>
                        <tr class="wind">
                            <th>tuuli</th>
                            {wind}
                        </tr>
                        <tr class="temperature">
                            <th>sade&<br/>mahd.</th>
                            {precipitation}
                        </tr>
                    </table>
                </div>
            </Suspense>
        </div>
    }
}

#[component]
pub fn Times() -> impl IntoView {
    let timestamp = use_timestamp();
    let timezone = Local::now().timezone();
    let seconds_since_epoch = Memo::new(move |_| (timestamp.get() / 1000.0) as i64);
    let local_now = Memo::new(move |_| DateTime::from_timestamp(seconds_since_epoch.get(), 0).unwrap().with_timezone(&timezone));
    let local_now_string = move || local_now.get().to_string();

    let fr_date = Memo::new(move |_| FRDateTime::from_timestamp(FRTimestamp::from_unix(local_now.get().timestamp())));
    let fr_time = Memo::new(move |_| {
        let fr_seconds_from_midnight = ((local_now.get().time().num_seconds_from_midnight() as f64) * (1000.0 / 864.0)).floor() as usize;
        let fr_hours = fr_seconds_from_midnight / (100 * 100);
        let fr_seconds_from_hour = fr_seconds_from_midnight - (100 * 100 * fr_hours);
        let fr_minutes = fr_seconds_from_hour / 100;
        let fr_seconds = fr_seconds_from_hour - (100 * fr_minutes);
        let fr_time = format!("{:02}:{:02}:{:02}", fr_hours, fr_minutes, fr_seconds);
        fr_time
    });
    let fr_date_time = move || format!("{}, {}", fr_date.get().to_string_default(), fr_time.get());

    let fi_date_time = move || local_now.get().format_localized("%A %e %B %Y, %H.%M.%S", chrono::Locale::fi_FI).to_string();

    view! {
        <div class="datetime">
            <p class="unix">{seconds_since_epoch}</p>
            <p class="iso">{local_now_string}</p>
            <p class="fr">{fr_date_time}</p>
            <p class="fi">{fi_date_time}</p>
        </div>
    }
}


#[component]
pub fn App() -> impl IntoView {
    // Provides context that manages stylesheets, titles, meta tags, etc.
    provide_meta_context();

    view! {
        // <Html attr:lang="en" attr:dir="ltr" attr:data-theme="light" />
        <Html attr:lang="en" attr:dir="ltr" />

        // sets the document title
        <Title text="homepage.juustodiilerit.fi" />

        // injects metadata in the <head> of the page
        <Meta charset="UTF-8" />
        <Meta name="viewport" content="width=device-width, initial-scale=1.0" />

        <header>
            <h1><span>homepage</span><span>.juustodiilerit</span><span>.fi</span></h1>
        </header>
        <main class="main-width">
            <Times/>
            <Weather/>
            <ul class="links">
                <li>
                    <a href="https://forgejo.juustodiilerit.fi">git (forgejo)</a>
                </li>
                <li>
                    <a href="https://mail.juustodiilerit.fi">mail</a>
                </li>
                <li>
                    <a href="https://passbolt.juustodiilerit.fi">passbolt</a>
                </li>
            </ul>
        </main>
    }
}
